/*
 * 12/06/2004
 *
 * AssemblerX86TokenMaker.java - An object that can take a chunk of text and
 *                             return a linked list of <code>Token</code>s
 *                             representing X86 assembler.
 * Copyright (C) 2004 Robert Futrell
 * email@address.com
 * www.website.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package org.fife.ui.rsyntaxtextarea;

import java.io.*;
import java.util.HashMap;
import javax.swing.text.Segment;


/**
 * This class takes plain text and returns tokens representing x86
 * assembler.<p>
 *
 * This implementation was created using
 * <a href="http://www.jflex.de/">JFlex</a> 1.4_5; however, the generated file
 * was modified for performance.  Memory allocation needs to be almost
 * completely removed to be competitive with the handwritten lexers (subclasses
 * of <code>AbstractTokenMaker</code>, so this class has been modified so that
 * Strings are never allocated (via yytext()), and the scanner never has to
 * worry about refilling its buffer (needlessly copying chars around).
 * We can achieve this because RText always scans exactly 1 line of tokens at a
 * time, and hands the scanner this line as an array of characters (a Segment
 * really).  Since tokens contain pointers to char arrays instead of Strings
 * holding their contents, there is no need for allocating new memory for
 * Strings.<p>
 *
 * The actual algorithm generated for scanning has, of course, not been
 * modified.<p>
 *
 * If you wish to regenerate this file yourself, keep in mind the following:
 * <ul>
 *   <li>The generated AssemblerX86TokenMaker.java</code> file will contain two
 *       definitions of both <code>yy_refill</code> and <code>yyreset</code>.
 *       You should hand-delete the second of each definition (the ones
 *       generated by the lexer), as these generated methods modify the input
 *       buffer, which we'll never have to do.</li>
 *   <li>You should also change the declaration/definition of yy_buffer to NOT
 *       be initialized.  This is a needless memory allocation for us since we
 *       will be pointing the array somewhere else anyway.</li>
 *   <li>You should NOT call <code>yylex()</code> on the generated scanner
 *       directly; rather, you should use <code>getTokenList</code> as you would
 *       with any other <code>TokenMaker</code> instance.</li>
 * </ul>
 *
 * @author Robert Futrell
 * @version 0.1
 *
 */
%%

%class AssemblerX86TokenMaker
%extends AbstractJFlexTokenMaker
%implements TokenMaker
%unicode
%ignorecase
%type org.fife.ui.rsyntaxtextarea.Token


%{


/*****************************************************************************/


	/**
	 * Constructor.  We must have this here as JFLex does not generate a
	 * no parameter constructor.
	 */
	public AssemblerX86TokenMaker() {
		super();
	}


/*****************************************************************************/


	/**
	 * Adds the token specified to the current linked list of tokens.
	 *
	 * @param tokenType The token's type.
	 */
	private void addToken(int tokenType) {
		addToken(yy_startRead, yy_markedPos-1, tokenType);
	}


/*****************************************************************************/


	/**
	 * Adds the token specified to the current linked list of tokens.
	 *
	 * @param tokenType The token's type.
	 */
	private void addToken(int start, int end, int tokenType) {
		int so = start + offsetShift;
		addToken(yy_buffer, start,end, tokenType, so);
	}


/*****************************************************************************/


	/**
	 * Adds the token specified to the current linked list of tokens.
	 *
	 * @param array The character array.
	 * @param start The starting offset in the array.
	 * @param end The ending offset in the array.
	 * @param tokenType The token's type.
	 * @param startOffset The offset in the document at which this token
	 *                    occurs.
	 */
	public void addToken(char[] array, int start, int end, int tokenType, int startOffset) {
		super.addToken(array, start,end, tokenType, startOffset);
		yy_startRead = yy_markedPos;
	}


/*****************************************************************************/


	/**
	 * Returns the first token in the linked list of tokens generated
	 * from <code>text</code>.  This method must be implemented by
	 * subclasses so they can correctly implement syntax highlighting.
	 *
	 * @param text The text from which to get tokens.
	 * @param initialTokenType The token type we should start with.
	 * @param startOffset The offset into the document at which
	 *                    <code>text</code> starts.
	 * @return The first <code>Token</code> in a linked list representing
	 *         the syntax highlighted text.
	 */
	public Token getTokenList(Segment text, int initialTokenType, int startOffset) {

		resetTokenList();
		this.offsetShift = -text.offset + startOffset;

		// Start off in the proper state.
		int state = Token.NULL;
		switch (initialTokenType) {
			default:
				state = Token.NULL;
		}

		s = text;
		try {
			yyreset(yy_reader);
			yybegin(state);
			return yylex();
		} catch (IOException ioe) {
			ioe.printStackTrace();
			return new DefaultToken();
		}

	}


/*****************************************************************************/


	/**
	 * Refills the input buffer.
	 *
	 * @return      <code>true</code> if EOF was reached, otherwise
	 *              <code>false</code>.
	 * @exception   IOException  if any I/O-Error occurs.
	 */
	private boolean yy_refill() throws java.io.IOException {
		return yy_currentPos>=s.offset+s.count;
	}


/*****************************************************************************/


	/**
	 * Resets the scanner to read from a new input stream.
	 * Does not close the old reader.
	 *
	 * All internal variables are reset, the old input stream 
	 * <b>cannot</b> be reused (internal buffer is discarded and lost).
	 * Lexical state is set to <tt>YY_INITIAL</tt>.
	 *
	 * @param reader   the new input stream 
	 */
	public final void yyreset(java.io.Reader reader) throws java.io.IOException {
		// 's' has been updated.
		yy_buffer = s.array;
		/*
		 * We replaced the line below with the two below it because yy_refill
		 * no longer "refills" the buffer (since the way we do it, it's always
		 * "full" the first time through, since it points to the segment's
		 * array).  So, we assign yy_endRead here.
		 */
		//yy_startRead = yy_endRead = s.offset;
		yy_startRead = s.offset;
		yy_endRead = yy_startRead + s.count - 1;
		yy_currentPos = yy_markedPos = yy_pushbackPos = s.offset;
		yy_lexical_state = YYINITIAL;
		yy_reader = reader;
		yy_atBOL  = true;
		yy_atEOF  = false;
	}


/*****************************************************************************/

%}

Letter				= ([A-Za-z_])
Digit				= ([0-9])
Number				= ({Digit}+)

Identifier			= (({Letter}|{Digit})[^ \t\f\n\,\.\+\-\*\/\%\[\]]+)

UnclosedStringLiteral	= ([\"][^\"]*)
StringLiteral			= ({UnclosedStringLiteral}[\"])
UnclosedCharLiteral		= ([\'][^\']*)
CharLiteral			= ({UnclosedCharLiteral}[\'])

CommentBegin			= ([;])

LineTerminator			= (\n)
WhiteSpace			= ([ \t\f])

Label				= (({Letter}|{Digit})+[\:])

Operator				= ("+"|"-"|"*"|"/"|"%"|"^"|"|"|"&"|"~"|"!"|"="|"<"|">")

%%

/* Keywords */

<YYINITIAL> ".186"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".286"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".286P"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".287"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".386"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".386P"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".387"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".486"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".486P"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".586"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".586P"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".686"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".686P"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".8086"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".8087"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".ALPHA"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".BREAK"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".BSS"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".CODE"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".CONST"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".CONTINUE"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".CREF"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".DATA"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".DATA?"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".DOSSEG"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".ELSE"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".ELSEIF"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".ENDIF"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".ENDW"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".ERR"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".ERR1"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".ERR2"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".ERRB"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".ERRDEF"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".ERRDIF"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".ERRDIFI"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".ERRE"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".ERRIDN"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".ERRIDNI"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".ERRNB"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".ERRNDEF"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".ERRNZ"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".EXIT"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".FARDATA"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".FARDATA?"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".IF"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".K3D"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".LALL"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".LFCOND"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".LIST"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".LISTALL"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".LISTIF"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".LISTMACRO"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".LISTMACROALL"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".MMX"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".MODEL"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".MSFLOAT"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".NO87"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".NOCREF"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".NOLIST"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".NOLISTIF"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".NOLISTMACRO"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".RADIX"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".REPEAT"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".SALL"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".SEQ"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".SFCOND"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".STACK"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".STARTUP"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".TEXT"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".TFCOND"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".UNTIL"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".UNTILCXZ"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".WHILE"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".XALL"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".XCREF"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".XLIST"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> ".XMM"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "__FILE__"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "__LINE__"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "A16"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "A32"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "ADDR"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "ALIGN"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "ALIGNB"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "ASSUME"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "BITS"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "CARRY?"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "CATSTR"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "CODESEG"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "COMM"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "COMMENT"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "COMMON"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "DATASEG"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "DOSSEG"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "ECHO"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "ELSE"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "ELSEIF"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "ELSEIF1"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "ELSEIF2"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "ELSEIFB"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "ELSEIFDEF"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "ELSEIFE"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "ELSEIFIDN"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "ELSEIFNB"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "ELSEIFNDEF"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "END"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "ENDIF"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "ENDM"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "ENDP"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "ENDS"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "ENDSTRUC"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "EVEN"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "EXITM"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "EXPORT"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "EXTERN"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "EXTERNDEF"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "EXTRN"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "FAR"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "FOR"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "FORC"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "GLOBAL"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "GOTO"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "GROUP"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "HIGH"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "HIGHWORD"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "IEND"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "IF"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "IF1"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "IF2"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "IFB"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "IFDEF"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "IFDIF"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "IFDIFI"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "IFE"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "IFIDN"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "IFIDNI"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "IFNB"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "IFNDEF"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "IMPORT"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "INCBIN"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "INCLUDE"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "INCLUDELIB"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "INSTR"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "INVOKE"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "IRP"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "IRPC"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "ISTRUC"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "LABEL"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "LENGTH"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "LENGTHOF"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "LOCAL"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "LOW"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "LOWWORD"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "LROFFSET"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "MACRO"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "NAME"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "NEAR"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "NOSPLIT"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "O16"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "O32"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "OFFSET"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "OPATTR"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "OPTION"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "ORG"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "OVERFLOW?"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "PAGE"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "PARITY?"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "POPCONTEXT"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "PRIVATE"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "PROC"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "PROTO"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "PTR"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "PUBLIC"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "PURGE"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "PUSHCONTEXT"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "RECORD"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "REPEAT"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "REPT"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "SECTION"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "SEG"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "SEGMENT"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "SHORT"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "SIGN?"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "SIZE"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "SIZEOF"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "SIZESTR"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "STACK"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "STRUC"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "STRUCT"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "SUBSTR"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "SUBTITLE"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "SUBTTL"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "THIS"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "TITLE"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "TYPE"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "TYPEDEF"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "UNION"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "USE16"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "USE32"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "USES"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "WHILE"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "WRT"						{ addToken(Token.PREPROCESSOR); }
<YYINITIAL> "ZERO?"						{ addToken(Token.PREPROCESSOR); }

<YYINITIAL> "DB"						{ addToken(Token.FUNCTION); }
<YYINITIAL> "DW"						{ addToken(Token.FUNCTION); }
<YYINITIAL> "DD"						{ addToken(Token.FUNCTION); }
<YYINITIAL> "DF"						{ addToken(Token.FUNCTION); }
<YYINITIAL> "DQ"						{ addToken(Token.FUNCTION); }
<YYINITIAL> "DT"						{ addToken(Token.FUNCTION); }
<YYINITIAL> "RESB"						{ addToken(Token.FUNCTION); }
<YYINITIAL> "RESW"						{ addToken(Token.FUNCTION); }
<YYINITIAL> "RESD"						{ addToken(Token.FUNCTION); }
<YYINITIAL> "RESQ"						{ addToken(Token.FUNCTION); }
<YYINITIAL> "REST"						{ addToken(Token.FUNCTION); }
<YYINITIAL> "EQU"						{ addToken(Token.FUNCTION); }
<YYINITIAL> "TEXTEQU"						{ addToken(Token.FUNCTION); }
<YYINITIAL> "TIMES"						{ addToken(Token.FUNCTION); }
<YYINITIAL> "DUP"						{ addToken(Token.FUNCTION); }

<YYINITIAL> "BYTE"						{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "WORD"						{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "DWORD"						{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "FWORD"						{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "QWORD"						{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "TBYTE"						{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "SBYTE"						{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "TWORD"						{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "SWORD"						{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "SDWORD"						{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "REAL4"						{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "REAL8"						{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "REAL10"						{ addToken(Token.DATA_TYPE); }

/* Registers */
<YYINITIAL> "AL"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "BL"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "CL"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "DL"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "AH"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "BH"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "CH"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "DH"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "AX"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "BX"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "CX"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "DX"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "SI"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "DI"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "SP"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "BP"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "EAX"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "EBX"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "ECX"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "EDX"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "ESI"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "EDI"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "ESP"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "EBP"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "CS"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "DS"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "SS"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "ES"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "FS"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "GS"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "ST"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "ST0"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "ST1"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "ST2"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "ST3"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "ST4"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "ST5"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "ST6"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "ST7"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "MM0"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "MM1"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "MM2"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "MM3"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "MM4"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "MM5"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "MM6"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "MM7"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "XMM0"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "XMM1"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "XMM2"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "XMM3"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "XMM4"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "XMM5"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "XMM6"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "XMM7"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "CR0"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "CR2"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "CR3"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "CR4"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "DR0"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "DR1"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "DR2"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "DR3"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "DR4"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "DR5"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "DR6"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "DR7"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "TR3"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "TR4"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "TR5"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "TR6"						{ addToken(Token.VARIABLE); }
<YYINITIAL> "TR7"						{ addToken(Token.VARIABLE); }

/* Pentium III Instructions. */
<YYINITIAL> "AAA"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "AAD"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "AAM"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "AAS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "ADC"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "ADD"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "ADDPS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "ADDSS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "AND"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "ANDNPS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "ANDPS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "ARPL"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "BOUND"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "BSF"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "BSR"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "BSWAP"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "BT"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "BTC"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "BTR"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "BTS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CALL"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CBW"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CDQ"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CLC"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CLD"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CLI"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CLTS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CMC"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CMOVA"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CMOVAE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CMOVB"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CMOVBE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CMOVC"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CMOVE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CMOVG"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CMOVGE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CMOVL"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CMOVLE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CMOVNA"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CMOVNAE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CMOVNB"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CMOVNBE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CMOVNC"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CMOVNE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CMOVNG"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CMOVNGE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CMOVNL"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CMOVNLE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CMOVNO"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CMOVNP"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CMOVNS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CMOVNZ"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CMOVO"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CMOVP"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CMOVPE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CMOVPO"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CMOVS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CMOVZ"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CMP"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CMPPS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CMPS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CMPSB"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CMPSD"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CMPSS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CMPSW"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CMPXCHG"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CMPXCHGB"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "COMISS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CPUID"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CWD"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CWDE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CVTPI2PS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CVTPS2PI"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CVTSI2SS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CVTSS2SI"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CVTTPS2PI"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "CVTTSS2SI"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "DAA"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "DAS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "DEC"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "DIV"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "DIVPS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "DIVSS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "EMMS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "ENTER"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "F2XM1"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FABS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FADD"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FADDP"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FBLD"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FBSTP"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FCHS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FCLEX"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FCMOVB"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FCMOVBE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FCMOVE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FCMOVNB"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FCMOVNBE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FCMOVNE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FCMOVNU"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FCMOVU"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FCOM"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FCOMI"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FCOMIP"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FCOMP"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FCOMPP"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FCOS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FDECSTP"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FDIV"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FDIVP"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FDIVR"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FDIVRP"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FFREE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FIADD"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FICOM"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FICOMP"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FIDIV"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FIDIVR"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FILD"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FIMUL"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FINCSTP"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FINIT"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FIST"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FISTP"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FISUB"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FISUBR"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FLD1"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FLDCW"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FLDENV"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FLDL2E"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FLDL2T"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FLDLG2"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FLDLN2"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FLDPI"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FLDZ"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FMUL"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FMULP"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FNCLEX"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FNINIT"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FNOP"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FNSAVE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FNSTCW"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FNSTENV"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FNSTSW"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FPATAN"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FPREM"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FPREMI"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FPTAN"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FRNDINT"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FRSTOR"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FSAVE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FSCALE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FSIN"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FSINCOS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FSQRT"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FST"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FSTCW"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FSTENV"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FSTP"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FSTSW"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FSUB"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FSUBP"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FSUBR"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FSUBRP"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FTST"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FUCOM"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FUCOMI"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FUCOMIP"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FUCOMP"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FUCOMPP"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FWAIT"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FXAM"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FXCH"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FXRSTOR"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FXSAVE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FXTRACT"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FYL2X"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "FYL2XP1"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "HLT"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "IDIV"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "IMUL"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "IN"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "INC"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "INS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "INSB"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "INSD"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "INSW"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "INT"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "INTO"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "INVD"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "INVLPG"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "IRET"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "JA"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "JAE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "JB"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "JBE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "JC"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "JCXZ"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "JE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "JECXZ"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "JG"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "JGE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "JL"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "JLE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "JMP"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "JNA"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "JNAE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "JNB"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "JNBE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "JNC"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "JNE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "JNG"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "JNGE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "JNL"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "JNLE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "JNO"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "JNP"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "JNS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "JNZ"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "JO"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "JP"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "JPE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "JPO"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "JS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "JZ"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "LAHF"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "LAR"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "LDMXCSR"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "LDS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "LEA"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "LEAVE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "LES"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "LFS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "LGDT"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "LGS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "LIDT"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "LLDT"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "LMSW"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "LOCK"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "LODS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "LODSB"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "LODSD"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "LODSW"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "LOOP"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "LOOPE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "LOOPNE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "LOOPNZ"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "LOOPZ"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "LSL"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "LSS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "LTR"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "MASKMOVQ"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "MAXPS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "MAXSS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "MINPS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "MINSS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "MOV"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "MOVAPS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "MOVD"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "MOVHLPS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "MOVHPS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "MOVLHPS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "MOVLPS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "MOVMSKPS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "MOVNTPS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "MOVNTQ"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "MOVQ"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "MOVS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "MOVSB"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "MOVSD"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "MOVSS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "MOVSW"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "MOVSX"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "MOVUPS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "MOVZX"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "MUL"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "MULPS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "MULSS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "NEG"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "NOP"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "NOT"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "OR"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "ORPS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "OUT"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "OUTS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "OUTSB"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "OUTSD"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "OUTSW"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PACKSSDW"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PACKSSWB"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PACKUSWB"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PADDB"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PADDD"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PADDSB"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PADDSW"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PADDUSB"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PADDUSW"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PADDW"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PAND"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PANDN"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PAVGB"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PAVGW"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PCMPEQB"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PCMPEQD"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PCMPEQW"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PCMPGTB"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PCMPGTD"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PCMPGTW"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PEXTRW"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PINSRW"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PMADDWD"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PMAXSW"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PMAXUB"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PMINSW"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PMINUB"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PMOVMSKB"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PMULHUW"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PMULHW"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PMULLW"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "POP"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "POPA"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "POPAD"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "POPAW"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "POPF"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "POPFD"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "POPFW"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "POR"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PREFETCH"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PSADBW"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PSHUFW"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PSLLD"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PSLLQ"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PSLLW"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PSRAD"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PSRAW"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PSRLD"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PSRLQ"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PSRLW"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PSUBB"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PSUBD"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PSUBSB"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PSUBSW"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PSUBUSB"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PSUBUSW"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PSUBW"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PUNPCKHBW"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PUNPCKHDQ"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PUNPCKHWD"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PUNPCKLBW"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PUNPCKLDQ"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PUNPCKLWD"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PUSH"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PUSHA"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PUSHAD"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PUSHAW"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PUSHF"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PUSHFD"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PUSHFW"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "PXOR"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "RCL"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "RCR"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "RDMSR"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "RDPMC"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "RDTSC"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "REP"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "REPE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "REPNE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "REPNZ"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "REPZ"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "RET"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "RETF"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "RETN"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "ROL"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "ROR"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "RSM"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SAHF"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SAL"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SAR"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SBB"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SCAS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SCASB"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SCASD"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SCASW"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SETA"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SETAE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SETB"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SETBE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SETC"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SETE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SETG"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SETGE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SETL"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SETLE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SETNA"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SETNAE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SETNB"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SETNBE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SETNC"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SETNE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SETNG"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SETNGE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SETNL"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SETNLE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SETNO"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SETNP"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SETNS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SETNZ"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SETO"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SETP"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SETPE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SETPO"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SETS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SETZ"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SFENCE"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SGDT"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SHL"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SHLD"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SHR"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SHRD"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SHUFPS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SIDT"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SLDT"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SMSW"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SQRTPS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SQRTSS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "STC"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "STD"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "STI"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "STMXCSR"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "STOS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "STOSB"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "STOSD"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "STOSW"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "STR"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SUB"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SUBPS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SUBSS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SYSENTER"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "SYSEXIT"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "TEST"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "UB2"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "UCOMISS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "UNPCKHPS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "UNPCKLPS"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "WAIT"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "WBINVD"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "VERR"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "VERW"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "WRMSR"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "XADD"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "XCHG"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "XLAT"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "XLATB"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "XOR"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "XORPS"						{ addToken(Token.RESERVED_WORD); }


<YYINITIAL> {

	{LineTerminator}				{ addNullToken(); return firstToken; }

	{WhiteSpace}+					{ addToken(Token.WHITESPACE); }

	/* String/Character Literals. */
	{CharLiteral}					{ addToken(Token.LITERAL_CHAR); }
	{UnclosedCharLiteral}			{ addToken(Token.ERROR_CHAR); /*addNullToken(); return firstToken;*/ }
	{StringLiteral}				{ addToken(Token.LITERAL_STRING_DOUBLE_QUOTE); }
	{UnclosedStringLiteral}			{ addToken(Token.ERROR_STRING_DOUBLE); addNullToken(); return firstToken; }

	/* Labels. */
	{Label}						{ addToken(Token.PREPROCESSOR); }

	^%({Letter}|{Digit})*			{ addToken(Token.FUNCTION); }

	/* Comment Literals. */
	{CommentBegin}.*				{ addToken(Token.COMMENT_EOL); addNullToken(); return firstToken; }

	/* Operators. */
	{Operator}					{ addToken(Token.OPERATOR); }

	/* Numbers */
	{Number}						{ addToken(Token.LITERAL_NUMBER_DECIMAL_INT); }

	/* Ended with a line not in a string or comment. */
	<<EOF>>						{ addNullToken(); return firstToken; }

	/* Catch any other (unhandled) characters. */
	{Identifier}					{ addToken(Token.IDENTIFIER); }
	.							{ addToken(Token.IDENTIFIER); }

}
