/*
 * 02/25/2005
 *
 * SASTokenMaker.java - An object that can take a chunk of text and
 *                      return a linked list of <code>Token</code>s
 *                      representing it in SAS.
 * Copyright (C) 2005 Robert Futrell
 * email@address.com
 * www.website.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package org.fife.ui.rsyntaxtextarea;

import java.io.*;
import java.util.HashMap;
import javax.swing.text.Segment;


/**
 * This class generates tokens representing a text stream as SAS.<p>
 *
 * This implementation was created using
 * <a href="http://www.jflex.de/">JFlex</a> 1.4_5; however, the generated file
 * was modified for performance.  Memory allocation needs to be almost
 * completely removed to be competitive with the handwritten lexers (subclasses
 * of <code>AbstractTokenMaker</code>, so this class has been modified so that
 * Strings are never allocated (via yytext()), and the scanner never has to
 * worry about refilling its buffer (needlessly copying chars around).
 * We can achieve this because RText always scans exactly 1 line of tokens at a
 * time, and hands the scanner this line as an array of characters (a Segment
 * really).  Since tokens contain pointers to char arrays instead of Strings
 * holding their contents, there is no need for allocating new memory for
 * Strings.<p>
 *
 * The actual algorithm generated for scanning has, of course, not been
 * modified.<p>
 *
 * If you wish to regenerate this file yourself, keep in mind the following:
 * <ul>
 *   <li>The generated SASTokenMaker.java</code> file will contain two
 *       definitions of both <code>yy_refill</code> and <code>yyreset</code>.
 *       You should hand-delete the second of each definition (the ones
 *       generated by the lexer), as these generated methods modify the input
 *       buffer, which we'll never have to do.</li>
 *   <li>You should also change the declaration/definition of yy_buffer to NOT
 *       be initialized.  This is a needless memory allocation for us since we
 *       will be pointing the array somewhere else anyway.</li>
 *   <li>You should NOT call <code>yylex()</code> on the generated scanner
 *       directly; rather, you should use <code>getTokenList</code> as you would
 *       with any other <code>TokenMaker</code> instance.</li>
 * </ul>
 *
 * @author Robert Futrell
 * @version 0.1
 *
 */
%%

%class SASTokenMaker
%extends AbstractJFlexTokenMaker
%implements TokenMaker
%unicode
%ignorecase
%type org.fife.ui.rsyntaxtextarea.Token


%{


/*****************************************************************************/


	/**
	 * Constructor.  This must be here because JFlex does not generate a
	 * no-parameter constructor.
	 */
	public SASTokenMaker() {
		super();
	}


/*****************************************************************************/


	/**
	 * Adds the token specified to the current linked list of tokens.
	 *
	 * @param tokenType The token's type.
	 */
	private void addToken(int tokenType) {
		addToken(yy_startRead, yy_markedPos-1, tokenType);
	}


/*****************************************************************************/


	/**
	 * Adds the token specified to the current linked list of tokens.
	 *
	 * @param tokenType The token's type.
	 */
	private void addToken(int start, int end, int tokenType) {
		int so = start + offsetShift;
		addToken(yy_buffer, start,end, tokenType, so);
	}


/*****************************************************************************/


	/**
	 * Adds the token specified to the current linked list of tokens.
	 *
	 * @param array The character array.
	 * @param start The starting offset in the array.
	 * @param end The ending offset in the array.
	 * @param tokenType The token's type.
	 * @param startOffset The offset in the document at which this token
	 *                    occurs.
	 */
	public void addToken(char[] array, int start, int end, int tokenType, int startOffset) {
		super.addToken(array, start,end, tokenType, startOffset);
		yy_startRead = yy_markedPos;
	}


/*****************************************************************************/


	/**
	 * Returns the first token in the linked list of tokens generated
	 * from <code>text</code>.  This method must be implemented by
	 * subclasses so they can correctly implement syntax highlighting.
	 *
	 * @param text The text from which to get tokens.
	 * @param initialTokenType The token type we should start with.
	 * @param startOffset The offset into the document at which
	 *                    <code>text</code> starts.
	 * @return The first <code>Token</code> in a linked list representing
	 *         the syntax highlighted text.
	 */
	public Token getTokenList(Segment text, int initialTokenType, int startOffset) {

		resetTokenList();
		this.offsetShift = -text.offset + startOffset;

		// Start off in the proper state.
		int state = Token.NULL;
		switch (initialTokenType) {
			case Token.LITERAL_STRING_DOUBLE_QUOTE:
				state = STRING;
				start = text.offset;
				break;
			case Token.LITERAL_CHAR:
				state = CHAR;
				start = text.offset;
				break;
			case Token.COMMENT_MULTILINE:
				state = MLC;
				start = text.offset;
				break;
			default:
				state = Token.NULL;
		}

		s = text;
		try {
			yyreset(yy_reader);
			yybegin(state);
			return yylex();
		} catch (IOException ioe) {
			ioe.printStackTrace();
			return new DefaultToken();
		}

	}


/*****************************************************************************/


	/**
	 * Refills the input buffer.
	 *
	 * @return      <code>true</code> if EOF was reached, otherwise
	 *              <code>false</code>.
	 * @exception   IOException  if any I/O-Error occurs.
	 */
	private boolean yy_refill() throws java.io.IOException {
		return yy_currentPos>=s.offset+s.count;
	}


/*****************************************************************************/


	/**
	 * Resets the scanner to read from a new input stream.
	 * Does not close the old reader.
	 *
	 * All internal variables are reset, the old input stream 
	 * <b>cannot</b> be reused (internal buffer is discarded and lost).
	 * Lexical state is set to <tt>YY_INITIAL</tt>.
	 *
	 * @param reader   the new input stream 
	 */
	public final void yyreset(java.io.Reader reader) throws java.io.IOException {
		// 's' has been updated.
		yy_buffer = s.array;
		/*
		 * We replaced the line below with the two below it because yy_refill
		 * no longer "refills" the buffer (since the way we do it, it's always
		 * "full" the first time through, since it points to the segment's
		 * array).  So, we assign yy_endRead here.
		 */
		//yy_startRead = yy_endRead = s.offset;
		yy_startRead = s.offset;
		yy_endRead = yy_startRead + s.count - 1;
		yy_currentPos = yy_markedPos = yy_pushbackPos = s.offset;
		yy_lexical_state = YYINITIAL;
		yy_reader = reader;
		yy_atBOL  = true;
		yy_atEOF  = false;
	}


/*****************************************************************************/

%}

LineTerminator		= ([\n])
Letter			= ([A-Za-z_])
Digit			= ([0-9])
Whitespace		= ([ \t]+)
Semicolon	= ([;])

Identifier		= (({Letter}|{Digit})+)
MacroVariable		= (&{Identifier})

Operators1		= ("+"|"-"|"*"|"/"|"^"|"|")
Operators2		= (([\^\~]?=)|(">"[=]?)|("<"[=]?))
Operators3		= ("eq"|"ne"|"gt"|"lt"|"ge"|"le"|"in")
Operator			= ({Operators1}|{Operators2}|{Operators3})
Separator			= ([\(\)])

StringBoundary		= (\")
CharBoundary		= (\')

LineCommentBegin	= ("*")
MLCBegin			= ("/*")
MLCEnd			= ("*/")

%state STRING
%state CHAR
%state MLC

%%

/* Keywords */
<YYINITIAL> "_all_"				{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "_character_"				{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "_data_"				{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "_infile_"				{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "_last_"				{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "_null_"				{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "_numeric_"				{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "_page_"				{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "_temporary_"				{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "abend"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "abort"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "all"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "alter"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "and"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "array"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "as"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "ascending"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "attrib"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "axis"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "bell"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "blank"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "border"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "bounds"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "by"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "call"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "cancel"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "cards"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "cards4"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "choro"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "class"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "classes"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "clear"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "close"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "compute"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "contrast"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "coord"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "coordinates"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "cov"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "create"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "data"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "datalines"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "datalines4"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "delete"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "descending"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "describe"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "discrete"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "disk"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "display"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "dm"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "do"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "drop"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "dummy"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "else"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "end"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "endrsubmit"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "endsas"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "error"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "except"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "expandtabs"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "factors"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "file"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "filename"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "flowover"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "footnote"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "frame"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "freq"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "from"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "go"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "goption"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "goptions"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "goto"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "grid"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "group"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "groupby"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "groupformat"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "having"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "haxis"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "hbar"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "heading"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "high"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "html"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "id"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "if"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "infile"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "informat"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "inner"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "input"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "insert"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "intersect"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "keep"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "keylabel"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "label"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "lable"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "legend"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "length"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "libname"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "lineqs"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "link"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "list"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "listing"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "log"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "lostcard"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "low"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "mark"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "matings"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "mean"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "merge"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "missing"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "missover"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "mod"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "model"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "modify"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "n"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "nocell"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "nocharacters"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "nodupkey"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "noexpandtabs"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "noframe"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "noheading"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "noinput"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "nolegend"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "nopad"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "noprint"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "nosharebuffers"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "not"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "note"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "notitle"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "notitles"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "notsorted"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "ods"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "old"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "option"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "or"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "order"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "orderby"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "other"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "otherwise"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "outer"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "output"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "over"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "overlay"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "overprint"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "pad"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "pageby"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "pagesize"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "parmcards"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "parmcards4"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "parms"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "pattern"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "pct"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "pctn"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "pctsum"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "picture"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "pie"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "pie3d"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "plotter"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "predict"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "prefix"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "printer"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "proc"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "ps"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "put"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "quit"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "random"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "range"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "remove"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "rename"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "response"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "replace"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "reset"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "retain"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "return"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "rsubmit"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "run"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "s2"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "select"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "set"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "sharebuffers"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "signoff"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "signon"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "sim"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "skip"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "source2"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "startsas"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "std"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "stop"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "stopover"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "strata"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "sum"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "sumby"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "supvar"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "symbol"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "table"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "tables"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "tape"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "terminal"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "test"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "then"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "time"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "title"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "to"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "transform"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "treatments"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "truncover"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "unbuf"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "unbuffered"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "union"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "until"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "update"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "validate"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "value"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "var"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "variables"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "vaxis"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "vbar"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "weight"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "when"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "where"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "while"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "with"						{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "window"					{ addToken(Token.RESERVED_WORD); }
<YYINITIAL> "x"						{ addToken(Token.RESERVED_WORD); }

/* Base SAS procs. */
<YYINITIAL> "append"			{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "calendar"			{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "catalog"			{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "chart"			{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "cimport"			{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "compare"			{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "contents"			{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "copy"				{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "cpm"				{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "cport"			{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "datasets"			{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "display"			{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "explode"			{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "export"			{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "fontreg"			{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "format"			{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "forms"			{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "fslist"			{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "import"			{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "means"			{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "migrate"			{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "options"			{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "optload"			{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "optsave"			{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "plot"				{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "pmenu"			{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "print"			{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "printto"			{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "proto"			{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "prtdef"			{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "prtexp"			{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "pwencode"			{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "rank"				{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "registry"			{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "report"			{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "sort"				{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "sql"				{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "standard"			{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "summary"			{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "tabulate"			{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "template"			{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "timeplot"			{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "transpose"			{ addToken(Token.DATA_TYPE); }

/* SAS/STAT procs. */
<YYINITIAL> "corr"				{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "freq"				{ addToken(Token.DATA_TYPE); }
<YYINITIAL> "univariate"		{ addToken(Token.DATA_TYPE); }

/* Macros. */
<YYINITIAL> "%abort"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%bquote"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%by"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%cms"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%copy"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%display"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%do"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%else"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%end"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%eval"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%global"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%go"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%goto"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%if"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%inc"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%include"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%index"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%input"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%keydef"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%length"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%let"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%local"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%macro"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%mend"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%nrbquote"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%nrquote"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%nrstr"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%put"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%qscan"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%qsubstr"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%qsysfunc"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%quote"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%qupcase"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%scan"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%str"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%substr"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%superq"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%syscall"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%sysevalf"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%sysexec"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%sysfunc"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%sysget"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%sysprod"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%sysrput"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%then"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%to"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%tso"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%unquote"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%until"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%upcase"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%while"			{ addToken(Token.FUNCTION); }
<YYINITIAL> "%window"			{ addToken(Token.FUNCTION); }

<YYINITIAL> {

	{LineTerminator}				{ addNullToken(); return firstToken; }

	/* Comments. */
	/* Do comments before operators as "*" can signify a line comment as */
	/* well as an operator. */
	^[ \t]*{LineCommentBegin}		{
									// We must do this because of how we
									// abuse JFlex; since we return an entire
									// list of tokens at once instead of a
									// single token at a time, the "^" regex
									// character doesn't really work, so we must
									// check that we're at the beginning of a
									// line ourselves.
									start = yy_startRead;
									// Might not be any whitespace.
									if (yylength()>1) {
										addToken(yy_startRead,yy_markedPos-2, Token.WHITESPACE);
										yy_startRead = yy_markedPos-1;
									}
									// Remember:  yy_startRead may now be updated,
									// so we must check against 'start'.
									if (start==s.offset) {
										addToken(yy_startRead,yy_endRead, Token.COMMENT_EOL);
										addNullToken();
										return firstToken;
									}
									else {
										addToken(yy_startRead,yy_startRead, Token.OPERATOR);
									}
								}
	{MLCBegin}					{ start = yy_markedPos-2; yybegin(MLC); }

	/* Do operators before identifiers since some of them are words. */
	{Operator}					{ addToken(Token.OPERATOR); }
	{Separator}					{ addToken(Token.SEPARATOR); }

	{Identifier}					{ addToken(Token.IDENTIFIER); }
	{MacroVariable}				{ addToken(Token.VARIABLE); }
	{Semicolon}					{ addToken(Token.IDENTIFIER); }

	{Whitespace}					{ addToken(Token.WHITESPACE); }

	{StringBoundary}				{ start = yy_markedPos-1; yybegin(STRING); }
	{CharBoundary}					{ start = yy_markedPos-1; yybegin(CHAR); }

	<<EOF>>						{ addNullToken(); return firstToken; }

	/* Catch any other (unhandled) characters and flag them as OK;    */
	/* This will include "." from statements like "from lib.dataset". */
	.							{ addToken(Token.IDENTIFIER); }

}

<STRING> {

	[^\n\"]+						{}
	{LineTerminator}				{ addToken(start,yy_startRead-1, Token.LITERAL_STRING_DOUBLE_QUOTE); return firstToken; }
/*	{StringBoundary}{StringBoundary}	{} */
	{StringBoundary}				{ yybegin(YYINITIAL); addToken(start,yy_startRead, Token.LITERAL_STRING_DOUBLE_QUOTE); }
	<<EOF>>						{ addToken(start,yy_startRead-1, Token.LITERAL_STRING_DOUBLE_QUOTE); return firstToken; }

}

<CHAR> {

	[^\n\']+						{}
	{LineTerminator}				{ yybegin(YYINITIAL); addToken(start,yy_startRead-1, Token.LITERAL_CHAR); return firstToken; }
/*	{CharBoundary}{CharBoundary}		{} */
	{CharBoundary}					{ yybegin(YYINITIAL); addToken(start,yy_startRead, Token.LITERAL_CHAR); }
	<<EOF>>						{ addToken(start,yy_startRead-1, Token.LITERAL_CHAR); return firstToken; }

}

<MLC> {

	[^\n\*]+						{}
	{LineTerminator}				{ addToken(start,yy_startRead-1, Token.COMMENT_MULTILINE); return firstToken; }
	{MLCEnd}						{ yybegin(YYINITIAL); addToken(start,yy_startRead+1, Token.COMMENT_MULTILINE); }
	\*							{}
	<<EOF>>						{ addToken(start,yy_startRead-1, Token.COMMENT_MULTILINE); return firstToken; }

}
